function [X,Xscale]=dscale(X)
% DSCALE
% ------
%         [X,Xscale]=dscale(X) scales data to zero mean and variance 1.
%
% INPUTS:
%      X      - Data matrix
%               (dimension is # of data vectors in matrix * # of data points)
%
% OUTPUTS:
%      X      - Scaled data matrix
%      Xscale - Matrix containing sample mean (column 1) and standard 
%               deviation (column 2) for each data vector in X.
%
% See the function 'wrescale' on how to rescale the weights of the
% trained network
%
% Programmed by Magnus Norgaard, IAU/IMM Technical University of Denmark
% LastEditDate: July 16, 1996

[r,N]  = size(X);              % r = # of data vectors, N = # of data
Xscale = [mean(X')' std(X')']; % Col. 1 contains mean values, col 2 the std's
for k=1:r,
  X(k,:) = (X(k,:) - Xscale(k,1))/Xscale(k,2);
end
